import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/primitives/combos/simpson.dart';
import 'package:fis_measure/process/primitives/multi_method/multi_simpson_path.dart';
import 'package:fis_measure/process/primitives/simpson_path.dart';
import 'package:vid/us/vid_us_unit.dart';

import 'formulas/cardiac.dart';
import 'lv_study.dart';

class LvSimpsonCal extends LvStudyCalculatorBase<LvStudySimpson> {
  LvSimpsonCal(super.ref) {
    //
  }

  @override
  void calculate() {
    if (ref.feature == null) return;

    final feature = ref.feature!;

    restoreVals();

    feature.updateStringValue(
      ItemOutputMeta(
          ref.meta.description, ref.meta.description, VidUsUnit.None),
      "",
    );

    for (var output in ref.meta.outputs) {
      switch (output.name) {
        case MeasureTerms.LVEDV:
          updateLVEDV();
          break;
        case MeasureTerms.LVESV:
          updateLVESV();
          break;
        case MeasureTerms.SV:
          updateSV();
          break;
        case MeasureTerms.EF:
          updateEF();
          break;
        case MeasureTerms.FS:
          updatePercentFS();
          break;
        case MeasureTerms.SI:
          updateSI();
          break;
        case MeasureTerms.LVEDVI:
          updateLVEDVI();
          break;
        case MeasureTerms.LVESVI:
          updateLVESVI();
          break;
      }
    }
  }

  @override
  void updateLVEDV() {
    final value = _clacLvedv(ref.a2cLvedv, ref.a4cLvedv);
    if (value != null) {
      updateFloatValueByName(MeasureTerms.LVEDV, value, unit: VidUsUnit.cm3);
      v.lvedv = value;
    }
  }

  @override
  void updateLVESV() {
    final value = _clacLvedv(ref.a2cLvesv, ref.a4cLvesv);
    if (value != null) {
      updateFloatValueByName(MeasureTerms.LVESV, value, unit: VidUsUnit.cm3);
      v.lvesv = value;
    }
  }

  double? _clacLvedv(MultiSimpsonPath a2c, MultiSimpsonPath a4c) {
    if (a2c.feature == null || a4c.feature == null) {
      return null;
    }
    final a2cFeature = a2c.feature!;
    final a4cFeature = a4c.feature!;
    double a2cL = a2cFeature.centerLineLength;
    double a4cL = a4cFeature.centerLineLength;

    List<double> aDiameters =
        List.generate(SimpsonPath.splitterCount, (index) => 0);
    int index = 0;
    a4cFeature.horizontalSplitterLegths.forEach((key, value) {
      aDiameters[index++] = value;
    });

    List<double> bDiameters =
        List.generate(SimpsonPath.splitterCount, (index) => 0);
    index = 0;
    a2cFeature.horizontalSplitterLegths.forEach((key, value) {
      bDiameters[index++] = value;
    });
    final longDiameter = (a2cL + a4cL) / 2.0;
    final lvesv = CardiacFormulas.lvSimsonVolume(
        longDiameter, aDiameters, bDiameters, SimpsonPath.splitterCount);
    return lvesv;
  }
}

class LvSingleSimpsonCal extends LvStudyCalculatorBase<LvStudySingleSimpson> {
  LvSingleSimpsonCal(super.ref) {
    //
  }

  @override
  void calculate() {
    if (ref.feature == null) return;

    final feature = ref.feature!;

    restoreVals();

    feature.updateStringValue(
      ItemOutputMeta(
          ref.meta.description, ref.meta.description, VidUsUnit.None),
      "",
    );

    for (var output in ref.meta.outputs) {
      switch (output.name) {
        case MeasureTerms.LVEDV:
          updateLVEDV();
          break;
        case MeasureTerms.LVESV:
          updateLVESV();
          break;
        case MeasureTerms.SV:
          updateSV();
          break;
        case MeasureTerms.EF:
          updateEF();
          break;
        case MeasureTerms.SI:
          updateSI();
          break;
        case MeasureTerms.LVEDVI:
          updateLVEDVI();
          break;
        case MeasureTerms.LVESVI:
          updateLVESVI();
          break;
      }
    }

    /**
     * 
     <OutPut Id="LVEDV" IsEnabled="False" CalculationIDs="LVEDV(BP Simp)"/>
<OutPut Id="LVESV" IsEnabled="False" CalculationIDs="LVESV(BP Simp)"/>
<OutPut Id="SV" IsEnabled="False" CalculationIDs="SV(Mod Simp)"/>
<OutPut Id="EF" IsEnabled="False" CalculationIDs="EF(Mod Simp)"/>
<OutPut Id="SI" IsEnabled="False" CalculationIDs="SI(Mod Simp)"/>
<OutPut Id="LVEDVI" IsEnabled="False" CalculationIDs="LVEDVI(Mod Simp)"/>
<OutPut Id="LVESVI" IsEnabled="False" CalculationIDs="LVESVI(Mod Simp)"/>
<OutPut Id="CO" IsEnabled="False" CalculationIDs="CO(Mod Simp)"/>
<OutPut Id="CI" IsEnabled="False" CalculationIDs="CI(Mod Simp)"/>
     */
  }

  @override
  void updateLVEDV() {
    final value = _calcLvedv(ref.lvedv);
    if (value != null) {
      updateFloatValueByName(MeasureTerms.LVEDV, value, unit: VidUsUnit.cm3);
      v.lvedv = value;
    }
  }

  @override
  void updateLVESV() {
    final value = _calcLvedv(ref.lvesv);
    if (value != null) {
      updateFloatValueByName(MeasureTerms.LVESV, value, unit: VidUsUnit.cm3);
      v.lvesv = value;
    }
  }

  double? _calcLvedv(MultiSimpsonPath item) {
    if (item.feature == null) {
      return null;
    }

    final feature = item.feature!;

    double longDiameter = roundDouble(feature.centerLineLength);
    int index = 0;
    List<double> bDiameters =
        List.generate(SimpsonPath.splitterCount, (index) => 0);
    feature.horizontalSplitterLegths.forEach((key, value) {
      bDiameters[index++] = value;
    });

    return CardiacFormulas.lvSimsonVolume(
      longDiameter,
      bDiameters,
      bDiameters,
      SimpsonPath.splitterCount,
    );
  }
}
